package pl.net.linux.krzak.kchat.util;
/*****************************/
/* kChat v2.0pre4	     */
/* krzak@linux.net.pl        */
/* Marcin Krzyzanowski       */
/* http://krzak.linux.net.pl */
/* License: GPL              */
/*****************************/
/* Copyright (C) 2001  Marcin Krzyzanowski <krzak@linux.net.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class kButton extends Component {

  private ActionListener actionListener;     // Post action events to listeners
  private String label;                      // The Button's text
  private String actionCommand;
  private boolean pressed = false; // true if the button is detented.
  private boolean over    = false; // true if the button is detented.
  
  
  /**
   * Constructs a kButton with no label.
   */
  public kButton() {
      this("");
  }

  /**
   * Constructs a kButton with the specified label.
   * @param label the label of the button
   */
  public kButton(String label) {
      this.label = label;
      this.actionCommand = label;
      enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }

  /**
   * gets the label
   */
  public String getLabel() {
      return label;
  }
  
  /**
   * sets the label
   */
  public void setLabel(String label) {
      this.label = label;
      invalidate();
      repaint();
  }
  
  /**
   * paints the kButton
   */
  public void paint(Graphics g) {
      // paint the interior of the button
      if (over)  g.setColor(getBackground().darker());
      if (pressed)  g.setColor(getBackground().darker().darker());
      
      if ((!pressed) && (!over)) g.setColor(getBackground());
	  
      g.fillRect(0, 0, getSize().width-2, getSize().height-3);
      
      // draw the perimeter of the button
      g.setColor(getForeground().darker());
      g.drawRect(0, 0, getSize().width-1, getSize().height-2);
      
      // draw the label centered in the button
      Font f = getFont();
      if(f != null) {
    	  int sx = getSize().width/2;
	  FontMetrics fm = getFontMetrics(getFont());
	  g.setColor(getForeground().brighter());
	  g.drawString(label,
		       sx - fm.stringWidth(label)/2,
		       fm.getHeight());
      }
  }
  
  /**
   * The preferred size of the button. 
   */
  public Dimension getPreferredSize() {
      Font f = getFont();
      if(f != null) {
	  FontMetrics fm = getFontMetrics(getFont());
	  return new Dimension((fm.stringWidth(label)+5), (fm.getHeight()+5));
      } else {
	  return new Dimension(100,100);
      }
  }
  
  /**
   * The minimum size of the button. 
   */
  public Dimension getMinimumSize() {
      return new Dimension(100, 100);
  }

  /**
   * Adds the specified action listener to receive action events
   * from this button.
   * @param listener the action listener
   */
   public void addActionListener(ActionListener listener) {
       actionListener = AWTEventMulticaster.add(actionListener, listener);
       enableEvents(AWTEvent.MOUSE_EVENT_MASK);
   }
 
   /**
    * Removes the specified action listener so it no longer receives
    * action events from this button.
    * @param listener the action listener
    */
   public void removeActionListener(ActionListener listener) {
       actionListener = AWTEventMulticaster.remove(actionListener, listener);
   }

  /**
   * Determine if click was inside button.
   */
   public boolean contains(int x, int y) {
     int mx = getSize().width;
     int my = getSize().height;
    return((x<mx)&&(x>0)&&(y<my)&&(y>0));
   }
   
   /**
    * Paints the button and distribute an action event to all listeners.
    */
   public void processMouseEvent(MouseEvent e) {
       Graphics g;
       switch(e.getID()) {
          case MouseEvent.MOUSE_PRESSED:
	    // render myself inverted....
	    pressed = true;

            // Repaint might flicker a bit. To avoid this, you can use
            // double buffering (see the Gauge example).
	    repaint(); 
	    break;
          case MouseEvent.MOUSE_RELEASED:
	    if(actionListener != null) {
	       actionListener.actionPerformed(new ActionEvent(
		   this, ActionEvent.ACTION_PERFORMED, actionCommand));
	    }
	    // render myself normal again
	    if(pressed == true) {
		pressed = false;

                // Repaint might flicker a bit. To avoid this, you can use
                // double buffering (see the Gauge example).
		repaint();
	    }
	    break;
          case MouseEvent.MOUSE_ENTERED:
	    if (over == false) {
		over = true;
		repaint();
		}
	    break;
          case MouseEvent.MOUSE_EXITED:
	    if (over == true) { 
		over = false;
		repaint();
	    }
	    if(pressed == true) {
		pressed = false;
		repaint();
	    }
	    break;
       }
       super.processMouseEvent(e);
   }


    public void setActionCommand(String cmd) {
    actionCommand = cmd;
    }

   
}

